@@ -1,26 +1,6 @@
-#!@PERL@
-# Copyright (C) 1989-2020 Free Software Foundation, Inc.
-#
-# This file is part of groff.
-#
-# groff is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free
-# Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# groff is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#!/usr/bin/perl
 
 use strict;
-use warnings;
-
-(my $progname = $0) =~s @.*/@@;
-
 # runs groff in safe mode, that seems to be the default
 # installation now. That means that I have to fix all nice
 # features outside groff. Sigh.
@@ -28,27 +8,14 @@ use warnings;
 # of security holes.
 
 my $no_exec;
-
-if (grep(/^--help$/, @ARGV)) {
-	print "usage: mmroff [-x] [groff-option ...] [file ...]\n";
-	exit;
-}
-
-if (grep(/^--version$/, @ARGV)) {
-	print "mmroff (groff) @VERSION@\n";
-	exit;
-}
-
 # check for -x and remove it
 if (grep(/^-x$/, @ARGV)) {
 	$no_exec++;
 	@ARGV = grep(!/^-x$/, @ARGV);
 }
 
-# mmroff should always have -mm, but not twice
-@ARGV = grep(!/^-mm$/, @ARGV);
-my $check_macro = "groff -rRef=1 -z -mm @ARGV";
-my $run_macro = "groff -mm @ARGV";
+my $check_macro = "groff -rRef=1 -z @ARGV";
+my $run_macro = "groff  @ARGV";
 
 my (%cur, $rfilename, $max_height, $imacro, $max_width, @out, @indi);
 open(MACRO, "$check_macro 2>&1 |") || die "run $check_macro:$!";
@@ -98,17 +65,12 @@ while(<MACRO>) {
 }
 close(MACRO);
 
-sub Die {
-	print STDERR "$progname: fatal error: @_\n";
-	exit 1;
-}
 
 if ($rfilename) {
 	push(@out, ".nr pict*max-height $max_height\n") if defined $max_height;
 	push(@out, ".nr pict*max-width $max_width\n") if defined $max_width;
 
-	open(OUT, ">$rfilename")
-		or &Die("unable to create $rfilename:$!");
+	open(OUT, ">$rfilename") || "create $rfilename:$!";
 	print OUT '.\" references', "\n";
 	my $i;
 	for $i (@out) {
@@ -123,7 +85,7 @@ exit system($run_macro);
 sub print_index {
 	my ($f, $ind, $macro) = @_;
 
-	open(OUT, ">$f") or &Die("unable to create $f:$!");
+	open(OUT, ">$f") || "create $f:$!";
 	my $i;
 	for $i (sort @$ind) {
 		if ($macro) {
@@ -171,9 +133,3 @@ sub psbb {
 	}
 	close(IN);
 }
-
-
-1;
-# Local Variables:
-# mode: CPerl
-# End:
