@@ -18,13 +18,12 @@ You should have received a copy of the GNU General Public License along
 with groff; see the file COPYING.  If not, write to the Free Software
 Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#include <stdio.h>
+#include "lib.h"
+
 #include <ctype.h>
-#include <string.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <errno.h>
-#include "lib.h"
 #include "errarg.h"
 #include "error.h"
 #include "stringclass.h"
