@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001, 2002
    Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
@@ -20,7 +20,6 @@ with groff; see the file COPYING.  If not, write to the Free Software
 Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 #include "table.h"
-#include "htmlindicate.h"
 
 #define MAX_POINT_SIZE 99
 #define MAX_VERTICAL_SPACING 72
@@ -110,7 +109,7 @@ int table_input::get()
 	else {
 	  state = MIDDLE;
 	  if (c == '\0') {
-	    error("illegal input character code 0");
+	    error("invalid input character code 0");
 	    break;
 	  }
 	}
@@ -139,7 +138,7 @@ int table_input::get()
 	  current_lineno++;
 	}
 	else if (c == '\0') {
-	  error("illegal input character code 0");
+	  error("invalid input character code 0");
 	  break;
 	}
 	return c;
@@ -221,8 +220,6 @@ void process_input_file(FILE *fp)
       break;
     case HAD_TS:
       if (c == ' ' || c == '\n' || compatible_flag) {
-	printf(".if '\\*(.T'html' \\X(table-start(\n");
-	html_begin_suppress(0);
 	putchar('.');
 	putchar('T');
 	putchar('S');
@@ -244,16 +241,12 @@ void process_input_file(FILE *fp)
 	    fputs(".TE", stdout);
 	    while ((c = getc(fp)) != '\n') {
 	      if (c == EOF) {
-		printf(".if '\\*(.T'html' \\X(table-end(\n");
-		html_end_suppress(0);
 		putchar('\n');
 		return;
 	      }
 	      putchar(c);
 	    }
 	    putchar('\n');
-	    printf(".if '\\*(.T'html' \\X(table-end(\n");
-	    html_end_suppress(0);
 	    current_lineno++;
 	  }
 	}
@@ -1385,7 +1378,7 @@ table *process_data(table_input &in, format *f, options *opt)
 	    f = newf;
 	}
 	if (line.length() >= 3
-	    && line[0] == '.' && line[1] == 'f' && line[2] == 'f') {
+	    && line[0] == '.' && line[1] == 'l' && line[2] == 'f') {
 	  line += '\0';
 	  interpret_lf_args(line.contents() + 3);
 	}
