@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001, 2002
    Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
@@ -281,9 +281,9 @@ protected:
   unsigned char unformat;
 public:
   hmotion_node(hunits i, node *next = 0)
-  : node(next), n(i), was_tab(0), unformat(0) {}
+    : node(next), n(i), was_tab(0), unformat(0) {}
   hmotion_node(hunits i, int flag1, int flag2, node *next = 0)
-  : node(next), n(i), was_tab(flag1), unformat(flag2) {}
+    : node(next), n(i), was_tab(flag1), unformat(flag2) {}
   node *copy();
   int reread(int *);
   int set_unformat_flag();
@@ -479,10 +479,11 @@ class suppress_node : public node {
   int emit_limits;	// must we issue the extent of the area written out?
   symbol filename;
   char position;
+  int  image_id;
 public:
   suppress_node(int, int);
-  suppress_node(symbol f, char p);
-  suppress_node(int, int, symbol f, char p);
+  suppress_node(symbol f, char p, int id);
+  suppress_node(int, int, symbol f, char p, int id);
   node *copy();
   void tprint(troff_output_file *);
   hunits width();
@@ -493,6 +494,30 @@ private:
   void put(troff_output_file *out, const char *s);
 };
 
+class glyph_color_node : public node {
+  color *c;
+public:
+  glyph_color_node(color *col);
+  node *copy();
+  void tprint(troff_output_file *);
+  hunits width();
+  int same(node *);
+  const char *type();
+  int force_tprint();
+};
+
+class fill_color_node : public node {
+  color *c;
+public:
+  fill_color_node(color *col);
+  node *copy();
+  void tprint(troff_output_file *);
+  hunits width();
+  int same(node *);
+  const char *type();
+  int force_tprint();
+};
+
 struct hvpair {
   hunits h;
   vunits v;
@@ -582,3 +607,4 @@ public:
 };
 
 font_family *lookup_family(symbol);
+symbol get_font_name(int, environment *);
